#include "AStar.h"

void AStar::swap(int idx1, int idx2)
{
	pAStarNode tmp = open_table[idx1];
	open_table[idx1] = open_table[idx2];
	open_table[idx2] = tmp;
}

// ???  
//   
std::vector<Vec2i> AStar::getPossiblePlacements(Vec2i pos, int group_size)
{
	int offset = group_size / 2;
	if (group_size > 1 && offset == 0)
		offset = 1;
	int py = (int)(pos.x);
	int px = (int)(pos.y);

	std::vector<Vec2i>positions;
	// loop through a block of the grid. offset on either side / top / bottom
	// add non barrier cells to open list
	for (int y = py - offset; y <= py + offset; y++)
	{
		if (y < 0 || y >= ROW_SIZE)
			continue;
		for (int x = px - offset; x <= px + offset; x++)
		{
			if (x < 0 || x >= COL_SIZE)
				continue;
			if (map_maze[x][y].s_style != BARRIER && map_maze[x][y].s_objSpace != 1)
			{
				positions.push_back(Vec2i(map_maze[x][y].s_y, map_maze[x][y].s_x));
			}
		}

	}
	return positions;
}
void AStar::adjust_heap(int /*i*/nIndex)
{
	int curr = nIndex;
	int child = curr * 2 + 1;   // ?????idx( ???0?????????curr*2+1 )  
	int parent = (curr - 1) / 2;  // ????idx  

	if (nIndex < 0 || nIndex >= open_node_count)
	{
		return;
	}

	// ????( ????????cuur parent )  
	//   
	while (child < open_node_count)
	{
		// ????????????  
		//   
		if (child + 1 < open_node_count && open_table[child]->s_g + open_table[child]->s_h > open_table[child + 1]->s_g + open_table[child + 1]->s_h)
		{
			++child; // ????????  
		}

		if (open_table[curr]->s_g + open_table[curr]->s_h <= open_table[child]->s_g + open_table[child]->s_h)
		{
			break;
		}
		else
		{
			swap(child, curr);            // ????  
			curr = child;               // ?????????  
			child = curr * 2 + 1;           // ??????  
		}
	}

	if (curr != nIndex)
	{
		return;
	}

	// ????( ?????cuur child?parent )  
	//   
	while (curr != 0)
	{
		if (open_table[curr]->s_g + open_table[curr]->s_h >= open_table[parent]->s_g + open_table[parent]->s_h)
		{
			break;
		}
		else
		{
			swap(curr, parent);
			curr = parent;
			parent = (curr - 1) / 2;
		}
	}
}

// ???????????open?  
//   
void AStar::insert_to_opentable(int x, int y, pAStarNode curr_node, pAStarNode end_node, int w)
{
	int i;

	if (map_maze[x][y].s_style != BARRIER && map_maze[x][y].s_objSpace != 1)        // ?????  
	{
		if (!map_maze[x][y].s_is_in_closetable)   // ?????  
		{
			if (map_maze[x][y].s_is_in_opentable) // ?open??  
			{
				// ???????????????  
				//   
				if (map_maze[x][y].s_g > curr_node->s_g + w)    // ?????  
				{
					map_maze[x][y].s_g = curr_node->s_g + w;
					map_maze[x][y].s_parent = curr_node;

					for (i = 0; i < open_node_count; ++i)
					{
						if (open_table[i]->s_x == map_maze[x][y].s_x && open_table[i]->s_y == map_maze[x][y].s_y)
						{
							break;
						}
					}

					adjust_heap(i);                   // ?????  
				}
			}
			else                                    // ??open?  
			{
				map_maze[x][y].s_g = curr_node->s_g + w;
				map_maze[x][y].s_h = abs(end_node->s_x - x) + abs(end_node->s_y - y);
				map_maze[x][y].s_parent = curr_node;
				map_maze[x][y].s_is_in_opentable = 1;
				open_table[open_node_count++] = &(map_maze[x][y]);
			}
		}
	}
}

// ????  
// ?????8???????  
//    
void AStar::get_neighbors(pAStarNode curr_node, pAStarNode end_node)
{
	int x = curr_node->s_x;
	int y = curr_node->s_y;

	// ????8????????  
	//   
	if ((x + 1) >= 0 && (x + 1) < ROW_SIZE && y >= 0 && y < COL_SIZE)
	{
		insert_to_opentable(x + 1, y, curr_node, end_node, 10);
	}

	if ((x - 1) >= 0 && (x - 1) < ROW_SIZE && y >= 0 && y < COL_SIZE)
	{
		insert_to_opentable(x - 1, y, curr_node, end_node, 10);
	}

	if (x >= 0 && x < ROW_SIZE && (y + 1) >= 0 && (y + 1) < COL_SIZE)
	{
		insert_to_opentable(x, y + 1, curr_node, end_node, 10);
	}

	if (x >= 0 && x < ROW_SIZE && (y - 1) >= 0 && (y - 1) < COL_SIZE)
	{
		insert_to_opentable(x, y - 1, curr_node, end_node, 10);
	}

	if ((x + 1) >= 0 && (x + 1) < ROW_SIZE && (y + 1) >= 0 && (y + 1) < COL_SIZE)
	{
		insert_to_opentable(x + 1, y + 1, curr_node, end_node, 14);
	}

	if ((x + 1) >= 0 && (x + 1) < ROW_SIZE && (y - 1) >= 0 && (y - 1) < COL_SIZE)
	{
		insert_to_opentable(x + 1, y - 1, curr_node, end_node, 14);
	}

	if ((x - 1) >= 0 && (x - 1) < ROW_SIZE && (y + 1) >= 0 && (y + 1) < COL_SIZE)
	{
		insert_to_opentable(x - 1, y + 1, curr_node, end_node, 14);
	}

	if ((x - 1) >= 0 && (x - 1) < ROW_SIZE && (y - 1) >= 0 && (y - 1) < COL_SIZE)
	{
		insert_to_opentable(x - 1, y - 1, curr_node, end_node, 14);
	}
}
void AStar::Reset()
{
	for (int c = 0; c < MAP_SIZE; c++)
	{
		close_table[c] = open_table[c] = path_stack[c] = nullptr;
	}
	open_node_count = close_node_count = 0;
	top = -1;
	pathPositions.resize(0);
	for (int i = 0; i < (int)ROW_SIZE - 1; i++)
	{
		for (int j = 0; j < (int)COL_SIZE - 1; j++)
		{
			map_maze[i][j].s_g = 0;
			map_maze[i][j].s_h = 0;
			map_maze[i][j].s_is_in_closetable = 0;
			map_maze[i][j].s_is_in_opentable = 0;
			map_maze[i][j].s_parent = NULL;
		}
	}
}
void AStar::Init(mapList& map)
{
	for (int c = 0; c < MAP_SIZE; c++)
	{
		close_table[c] = open_table[c] = path_stack[c] = nullptr;
	}
	open_node_count = close_node_count = 0;
	top = -1;
	pathPositions.resize(0);
	for (int i = 0; i < (int)map.size(); i++)
	{
		for (int j = 0; j < (int)map[0].size(); j++)
		{
			map_maze[i][j].s_g = 0;
			map_maze[i][j].s_h = 0;
			map_maze[i][j].s_is_in_closetable = 0;
			map_maze[i][j].s_is_in_opentable = 0;
			map_maze[i][j].s_style = map[i][j];
			map_maze[i][j].s_objSpace = 0;
			map_maze[i][j].s_x = i;
			map_maze[i][j].s_y = j;
			map_maze[i][j].s_parent = NULL;




		}
	}
}
AStar::AStar(int maze[M][N], int m, int n)
{

	int       i, j;

	// ?????  
	//   
	for (i = 0; i < m; ++i)
	{
		for (j = 0; j < n; ++j)
		{
			map_maze[i][j].s_g = 0;
			map_maze[i][j].s_h = 0;
			map_maze[i][j].s_is_in_closetable = 0;
			map_maze[i][j].s_is_in_opentable = 0;
			map_maze[i][j].s_style = maze[i][j];
			map_maze[i][j].s_x = i;
			map_maze[i][j].s_y = j;
			map_maze[i][j].s_parent = NULL;

			if (map_maze[i][j].s_style == STARTNODE)  // ??  
			{
				start_node = &(map_maze[i][j]);
			}
			else if (map_maze[i][j].s_style == ENDNODE)    // ??  
			{
				end_node = &(map_maze[i][j]);
			}


		}

		printf("\n");
	}
}

std::vector<Vec2i> AStar::findPath(Vec2i start, Vec2i end)
{
	// ???????  
	//   

	start_node = &(map_maze[start.y][start.x]);
	end_node = &(map_maze[end.y][end.x]);

	AStarNode *curr_node = nullptr;           // ???  
	int       is_found;         // ??????  
	std::vector<Vec2i>tmp;
	tmp.resize(0);
	pathPositions.clear();

	// ????A*??????  
	//    
	open_table[open_node_count++] = start_node;         // ?????open?  

	start_node->s_is_in_opentable = 1;               // ??open?  
	start_node->s_g = 0;
	start_node->s_h = abs(end_node->s_x - start_node->s_x) + abs(end_node->s_y - start_node->s_y);
	start_node->s_parent = NULL;

	if (start_node->s_x == end_node->s_x && start_node->s_y == end_node->s_y)
	{

		return tmp;
	}

	is_found = 0;

	while (1)
	{
		// for test  
		//   
		/*      for ( x = 0; x < open_node_count; ++x )
		{
		printf("(%d,%d):%d   ", open_table[x]->s_x, open_table[x]->s_y, open_table[x]->s_g+open_table[x]->s_h);
		}
		printf("\n\n");
		*/
		curr_node = open_table[0];      // open?????????f?????(????????)  
		open_table[0] = open_table[--open_node_count];  // ???????????????????  
		adjust_heap(0);               // ???  

		close_table[close_node_count++] = curr_node;    // ?????close?  
		curr_node->s_is_in_closetable = 1;       // ???close???  

		if (curr_node->s_x == end_node->s_x && curr_node->s_y == end_node->s_y)// ???close????  
		{
			is_found = 1;
			break;
		}

		get_neighbors(curr_node, end_node);           // ??????  

		if (open_node_count == 0)             // ??????  
		{
			is_found = 0;
			break;
		}
	}

	if (is_found == 1)
	{
		curr_node = end_node;

		while (curr_node)
		{
			path_stack[++top] = curr_node;
			if (curr_node)
			{
				float X = ((float)curr_node->s_x * map_TILE_WIDTH) + 16.0f;
				float Y = ((float)curr_node->s_y * map_TILE_WIDTH) + 16.0f;
				Vec2i p(Y, X);
				pathPositions.push_back(p);
			}
			curr_node = curr_node->s_parent;

		}
		//reverse order

		if (pathPositions.size() > 0)
		{

			for (int i = (int)(pathPositions.size() - 1); i >= 0; i--)
			{
				tmp.push_back(pathPositions[i]);
			}

		}
	}

	return tmp;

}