#include "Arena.h"
#include "Graphics.h"
PlayingField::PlayingField()
{
}

Block* PlayingField::GetCell(Vec2i pos)
{
	int col = (int)((float)pos.x - m_size.left) / m_cellSize.width ;
	int row =  (int)((float)pos.y - m_size.top) /  m_cellSize.height ;
	int index = row * m_columns + col;
	if (index < 0 || index >= m_grid.size())
		return nullptr;

	return &m_grid[(uint)index];
}

void PlayingField::Initialize(std::vector<std::string> level_data, Vec4f frame, Vec2i cellSize, Color fillColor, Color borderColor)
{
	m_grid.clear();
	m_size = frame;
	m_cellSize = cellSize;
	m_width = m_size.right - m_size.left;
	m_height = m_size.bottom - m_size.top;
	m_columns = m_width / m_cellSize.width;
	m_rows = m_height / m_cellSize.height;
	int startX = m_size.left;
	int startY = m_size.top;
	std::string currLine;
	for (int r = 0; r < m_rows; r++)
	{
		startX = m_size.left;
		currLine = level_data[r];
		for (int c = 0; c < m_columns; c++)
		{
			m_grid.push_back(Block(Vec4f(startX, startY, startX + m_cellSize.width, startY + m_cellSize.height),
				Color(55,55,55,55),Color(155,55,55,55)));
			
			if (currLine[c] == 'B')
			{
				m_grid.back().SetVisible(true);
				m_numbBlocks++;
			}
			startX += m_cellSize.width;
		}
		startY += m_cellSize.height;
	}

	m_fillColor   = fillColor;
	m_borderColor = borderColor;
}

void PlayingField::DrawFilled(Graphics & gfx)
{
	gfx.DrawSquareFilled(m_size, m_fillColor);
	
}

void PlayingField::DrawBordered(Graphics & gfx)
{
	gfx.DrawSquareBorder(m_size, m_borderColor);
	
}

void PlayingField::DrawFilledBordered(Graphics & gfx)
{
	DrawFilled(gfx);
	DrawBordered(gfx);
}

void PlayingField::DrawCellsBorder(Graphics & gfx)
{
	for (auto it : m_grid)
	{
		if (it.IsVisible())
		{
				gfx.DrawSquareBorder(*it.GetDimension(), m_borderColor);
		}
		

	}
}

void PlayingField::DrawCellsFilled(Graphics & gfx)
{
	for (auto it : m_grid)
	{
		gfx.DrawSquareFilled(*it.GetDimension(), m_fillColor);
	}
}

void PlayingField::DrawCellsFilledBorder(Graphics & gfx)
{
	DrawCellsFilled(gfx);
	DrawCellsBorder(gfx);
}

void PlayingField::ConfineAndRebound(Vec4f & sq, Vec2f & vel)
{
	m_hitBottom = false;
	float w = sq.right - sq.left;
	float h = sq.bottom - sq.top;
	bool moveX = false;
	bool moveY = false;
	if (sq.left < m_size.left)
	{
		sq.left = m_size.left;
		sq.right = sq.left + w;
		moveX = true;
	}if (sq.right > m_size.right)
	{
		sq.right = m_size.right;
		sq.left = sq.right - w;
		moveX = true;
	}if (sq.top < m_size.top)
	{
		sq.top = m_size.top;
		sq.bottom = sq.top + h;
		moveY = true;
	}if (sq.bottom > m_size.bottom)
	{
		sq.bottom = m_size.bottom;
		sq.top = sq.bottom - h;
		moveY = true;
		m_hitBottom = true;
	}
	if (moveX)vel.x = -vel.x;
	if (moveY)vel.y = -vel.y;
}

bool PlayingField::DoCollision(Ball * obj)
{
	std::vector<Vec2i> cells;
	bool hit = false;
	cells.emplace_back(Vec2i((int)obj->GetDimension()->left,
		(int)obj->GetDimension()->top));

	cells.emplace_back(Vec2i((int)obj->GetDimension()->right,
		(int)obj->GetDimension()->top));

	cells.emplace_back(Vec2i((int)obj->GetDimension()->left,
		(int)obj->GetDimension()->bottom));

	cells.emplace_back(Vec2i((int)obj->GetDimension()->left,
		(int)obj->GetDimension()->bottom));

	Block* m_lastBlock = nullptr;
	for (auto it : cells)
	{
		Block* block = GetCell(it);
		if (!block || m_lastBlock == block)
			continue;
		if (block->IsVisible())
		{
			Vec2f v = obj->Velocity();
			if (block->Collision(*obj,v))
			{
				obj->SetVelocity(v);
				block->SetVisible(false);
				m_lastBlock = block;
				m_numbBlocks--;
				if (m_numbBlocks < 0)
					m_numbBlocks = 0;
				hit = true;

			}
		}
	}
	

	return hit;
}

bool PlayingField::BlocksStillRemaining()
{
	// return whether or not numbBlocks
	// is greater than zero
	return m_numbBlocks > 0;
}

bool PlayingField::HitBottom()
{
	return m_hitBottom;
}
