#include "Block.h"
#include "Graphics.h"
Block::Block()
{

}
Block::Block(Vec4f size, Color color, Color border)
	:
	m_pos(size), m_fillColor(color),m_borderColor(border)
{

}
bool Block::IsVisible() { return m_visible; }

void Block::SetVisible(const bool val)
{
	m_visible = val;
}

Vec4f* Block::GetDimension()
{
	return &m_pos;
}

void Block::SetFillColor(Color c)
{
	m_fillColor = c;
}

Color Block::GetColor()
{
	return m_fillColor;
}

Vec2i Block::GetCenter()
{
	float w = m_pos.right  - m_pos.left;
	float h = m_pos.bottom - m_pos.top;
	w = w * 0.5f;
	h = h * 0.5f;

	Vec2i result((int)(m_pos.left + w), (int)(m_pos.top + h));
	return result;
	
}

float Block::Width()
{
	return m_pos.right - m_pos.left;
}

float Block::Height()
{
	return m_pos.bottom - m_pos.top;
}



bool Block::Collision(Block & other, Vec2f& vel)
{
	bool collided = false;
	if (m_pos.Overlaps(*other.GetDimension()))
	{
	
		collided = true;
		
		float dx = 0.0f;
		float dy = 0.0f;;

		if (vel.x > 0.0f)
		{
			dx = fabsf(other.GetDimension()->right - m_pos.left);
			
			if (vel.y > 0.0f)
			{
				dy = fabsf(other.GetDimension()->bottom - m_pos.top);
				
			}
			else
			{
			     dy = fabsf(other.GetDimension()->top - m_pos.bottom);
				 
			}
		}
		else// vel.x > 0
		{
			 dx = fabsf(other.GetDimension()->left - m_pos.right);
			
			if (vel.y > 0.0f)
			{
			   dy = fabsf(other.GetDimension()->bottom - m_pos.top);
			   
			}
			else
			{
				 dy = fabsf(other.GetDimension()->top - m_pos.bottom);
				
			}
		}
		
		if (dx > dy)
		{
			vel.y = -vel.y;
			other.GetDimension()->top += dy * sgn(vel.y);
			other.GetDimension()->bottom += dy * sgn(vel.y);
		}
		else
		{
			vel.x = -vel.x;
			other.GetDimension()->left += dx * sgn(vel.x);
			other.GetDimension()->right += dx * sgn(vel.x);
		}

	};
	return collided;
}

void Block::DrawFilled(Graphics& gfx)
{
	if (m_visible)
	{
		gfx.DrawSquareFilled(m_pos, m_fillColor);
		
	}
}

void Block::DrawBordered(Graphics & gfx)
{
    	gfx.DrawSquareBorder(m_pos, m_borderColor);
}


