/****************************************************************************************** 
 *	Chili DirectX Framework Version 16.07.20											  *	
 *	Game.cpp																			  *
 *	Copyright 2016 PlanetChili.net <http://www.planetchili.net>							  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "MainWindow.h"
#include "Game.h"

Game::Game( MainWindow& wnd )
	:
	wnd( wnd ),
	gfx( wnd ),
	m_paddle(Vec4f(360.0f, 540.0f, 440.0f, 550.0f), Color(0, 255, 0, 0), Color(0, 0, 255, 255)),
	m_mainBorder(Vec4f(40, 40, 760, 560), Color(255, 255, 255, 255), Color(255, 255, 255, 255))
{
	m_balls.emplace_back(Vec4f(390.0f, 522.0f, 410.0f, 542.0f), Color(0, 255, 0, 0), Color(255, 255, 255, 255));
	m_balls.emplace_back(Vec4f(40.0f, 565.0f, 60.0f, 585.0f), Color(0, 255, 0, 0), Color(255, 255, 255, 255));
	m_balls.emplace_back(Vec4f(70.0f, 565.0f, 90.0f, 585.0f), Color(0, 255, 0, 0), Color(255, 255, 255, 255));
	// set to always point at first one
	// we will delete the last one in the container
	// when we lose a ball
	m_ball = &m_balls[0];
	m_ballCount = m_balls.size();
	m_ball->SetSpeed(5.0f);
	m_ball->SetVelocity(Vec2f(0.0f,0.0f));
	m_ball->SetVisible(true);
	m_paddle.SetSpeed(6.0f);
	m_paddle.SetVelocity(Vec2f(1.0f, 0.0f));
	m_paddle.SetVisible(true);
	m_paddle.CaptureBall(m_ball);


	for (int c = 0; c < m_maxLevels; c++)
	{
		std::string fn = "..\\Engine\\";
		
		fn += ("Level_" + std::to_string(c) + ".txt");
		m_levelFilenames.push_back(fn);
	}
	for (int c = 0; c < 3; c++)
	{
		IMAGEINFO im;
		std::string fn = "..\\Engine\\";

		fn += ("image" + std::to_string(c+1) + ".bmp");
		im.filename = fn;
		if (LoadBitMapImage(im))
		{
			m_images.push_back(im);
		};
	}
	
	
}

void Game::Go()
{
	gfx.BeginFrame();	
	UpdateModel();
	ComposeFrame();
	gfx.EndFrame();
}

void Game::UpdateModel()
{
	
	//psudeo frame timer
	// 60 hz
	
	
	switch (m_gameState)
	{
	case Game_State::intro:
	{
		// wait untill enter key pressed
		if (wnd.kbd.KeyIsPressed(VK_RETURN))
			m_gameState = Game_State::load;
	}
		break;
	case Game_State::load:
		LoadNewLevel();
		
		timer = 0.0f;// reset timer
		m_gameState = Game_State::start;
		break;
	case Game_State::paused:
		if (wnd.kbd.KeyIsPressed(VK_RETURN))
			m_gameState = Game_State::running;

		break;
	case Game_State::start:
		timer += frame_time;
		*m_ball->GetDimension() = Vec4f(380.0f, 520.0f, 400.0f, 540.0f);
		*m_paddle.GetDimension() = Vec4f(360.0f, 540.0f, 440.0f, 550.0f);

		if (timer > 2.0f)// 3 second delay
		{
			m_gameState = Game_State::running;
			float prcnt = (float)(1 + m_currLevel) / 10.0f;
			
			m_paddle.SetVelocity(Vec2f(1.0f,0.0f));
			
			m_paddle.ReleaseBall();
		}

		break;
	case Game_State::over:
	{
		timer += frame_time;
		if (timer > 3.0f)
		{
			m_currLevel = 0;
			timer = 0.0f;
			m_gameState = Game_State::intro;
		}
	}
		break;
	case Game_State::exit:
		PostQuitMessage(0);
		break;
	case Game_State::running: // RUN TIME ///////////
	{
		HandleInput();
		HandleBall();
	
		
	}
	break;
	}
	}

void Game::ComposeFrame()
{
	int menuIndex = -1;
	m_mainBorder.DrawBordered(gfx);
	switch (m_gameState)
	{
	case Game_State::intro:
		gfx.DrawImage(m_images[0]);
		break;
	case Game_State::paused:
		gfx.DrawImage(m_images[1]);
		break;
	case Game_State::over:
		gfx.DrawImage(m_images[2]);
		break;
	case Game_State::running:
	case Game_State::start:
		m_field.DrawCellsBorder(gfx);

		for (auto it : m_balls)
			gfx.DrawCircle(it.GetCenter().x, it.GetCenter().y, it.Radius(), Color(255, 255, 255, 255));
			
		m_paddle.DrawBordered(gfx);
		break;
	}
	
}
bool Game::LoadNewLevel()
{
	std::ifstream ifs(m_levelFilenames[(uint)m_currLevel].c_str());
	std::vector<std::string> data;
	if (m_balls.size() == 0)
	{
		m_balls.emplace_back(Vec4f(390.0f, 520.0f, 410.0f, 540.0f), Color(0, 255, 0, 0), Color(255, 255, 255, 255));
		m_balls.emplace_back(Vec4f(40.0f, 565.0f, 60.0f, 585.0f), Color(0, 255, 0, 0), Color(255, 255, 255, 255));
		m_balls.emplace_back(Vec4f(70.0f, 565.0f, 90.0f, 585.0f), Color(0, 255, 0, 0), Color(255, 255, 255, 255));
		m_ball = &m_balls[0];
		m_ballCount = m_balls.size();
	}
	if (ifs.good())
	{
		std::string str;
		while (std::getline(ifs, str))
		{
			data.push_back(str);
		}
		ifs.close();
		m_field.Initialize(data, Vec4f(40, 40, 760, 560), Vec2i(40, 40),
			Color(255, 0, 255, 255), Color(155, 55, 55, 55));
		*m_ball->GetDimension() = Vec4f(390.0f, 520.0f, 410.0f, 540.0f);
		*m_paddle.GetDimension() = Vec4f(360.0f, 540.0f, 440.0f, 550.0f);

	}
	else
	{
		return false;
	}
	return true;
}

void Game::HandleInput()
{
	m_paddle.SetVelocity(Vec2f(0.0f,0.0f));
	float prcnt = (float)(1 + m_currLevel) / 10.0f;
	if (wnd.kbd.KeyIsPressed(VK_LEFT))
	{
		m_paddle.SetVelocity(Vec2f(-1.0f, 0.0f));
	}
	else if (wnd.kbd.KeyIsPressed(VK_RIGHT))
	{
		m_paddle.SetVelocity(Vec2f(1.0f, 0.0f));
	}

	m_paddle.Update();
	Vec2f v = m_paddle.Velocity();
	m_field.ConfineAndRebound(*m_paddle.GetDimension(),v );
	m_paddle.SetVelocity(v);
	if (wnd.kbd.KeyIsPressed(VK_SPACE))
	{
		 m_paddle.CaptureBall(m_ball);
	}
	else
	{
		if (m_paddle.BallCaptured())
		{
			m_paddle.ReleaseBall();
		}
		else
		{
			Vec2f v = m_ball->Velocity();
			m_paddle.Collision(*m_ball, v);
			m_ball->SetVelocity(v);
		}
	}

	
	
	if (wnd.kbd.KeyIsPressed(VK_ESCAPE))
	{
		m_gameState = Game_State::exit;
	}
	if (wnd.kbd.KeyIsPressed('P'))
	{

		if (m_gameState != Game_State::paused)
			m_gameState = Game_State::paused;
		
	}
	
}

void Game::HandleBall()
{
	m_ball->Update();
	if (m_field.DoCollision(m_ball))
	{
		if (!m_field.BlocksStillRemaining())
		{
			m_currLevel++;
			if(m_currLevel >= m_maxLevels)
				m_gameState = Game_State::over;
			else
			    m_gameState = Game_State::load;
		}
	};
	Vec2f v = m_ball->Velocity();
	m_field.ConfineAndRebound(*m_ball->GetDimension(), v);
	m_ball->SetVelocity(v);
	
	if (m_field.HitBottom())
	{
		m_balls.pop_back();
		m_ballCount = m_balls.size();
		m_gameState = Game_State::start;
		timer = 0.0f;
		if (m_ballCount == 0)
			m_gameState = Game_State::over;
	
	}
}

bool Game::LoadBitMapImage(IMAGEINFO& imageData)
{
	int error, imageSize, i, j, k, index;
	FILE* filePtr;
	unsigned long long count;
	BITMAPFILEHEADER bitmapFileHeader;
	BITMAPINFOHEADER bitmapInfoHeader;
	unsigned char* bitmapImage;


	// Open the color map file in binary.
	error = fopen_s(&filePtr, imageData.filename.c_str(), "rb");
	if (error != 0)
	{
		return false;
	}

	// Read in the file header.
	count = fread(&bitmapFileHeader, sizeof(BITMAPFILEHEADER), 1, filePtr);
	if (count != 1)
	{
		return false;
	}

	// Read in the bitmap info header.
	count = fread(&bitmapInfoHeader, sizeof(BITMAPINFOHEADER), 1, filePtr);
	if (count != 1)
	{
		return false;
	}

	

	// Calculate the size of the bitmap image data.
	// Since this is non-divide by 2 dimensions (eg. 257x257) need to add extra byte to each line.
	imageSize = bitmapInfoHeader.biHeight * ((bitmapInfoHeader.biWidth * 3)/*+1*/);// 
	imageData.width = (int)bitmapInfoHeader.biWidth;
	imageData.height = (int)bitmapInfoHeader.biHeight;
	imageData.columns = imageData.width / 3;
	imageData.rows = imageData.height;
	// Allocate memory for the bitmap image data.
	bitmapImage = new unsigned char[imageSize];
	if (!bitmapImage)
	{
		return false;
	}

	// Move to the beginning of the bitmap data.
	fseek(filePtr, bitmapFileHeader.bfOffBits, SEEK_SET);

	// Read in the bitmap image data.
	count = fread(bitmapImage, 1, imageSize, filePtr);
	if (count != imageSize)
	{
		if (bitmapImage)
			delete[] bitmapImage;
		return false;
	}

	// Close the file.
	error = fclose(filePtr);
	if (error != 0)
	{
		if(bitmapImage)
		  delete[] bitmapImage;
		return false;
	}

	// Initialize the position in the image data buffer.
	k = 0;

	// Read the image data 
	for (j = 0; j<bitmapInfoHeader.biHeight; j++)
	{
		for (i = 0; i<bitmapInfoHeader.biWidth; i++)
		{
			    // Bitmaps are upside down so load bottom to top.
				Color c = Color(255,// alpha
				bitmapImage[k + 2],//red
				bitmapImage[k + 1],// green
				bitmapImage[k]); // blue
			
			imageData.data.push_back(c);
			k += 3;
		}

		// Compensate for extra byte at end of each 
		// line in non-divide by 2 bitmaps (eg. 257x257).
		//k++;
	}

	// Release the bitmap image data.
	delete[] bitmapImage;
	bitmapImage = 0;

	

	return true;
	
}
