/****************************************************************************************** 
 *	Chili DirectX Framework Version 16.07.20											  *	
 *	Game.h																				  *
 *	Copyright 2016 PlanetChili.net <http://www.planetchili.net>							  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#pragma once

#include "Keyboard.h"
#include "Mouse.h"
#include "Graphics.h"
#include "Colors.h"
#include "Arena.h"
#include "Paddle.h"
#include <string>
#include <fstream>

class Game
{
public:
	Game( class MainWindow& wnd );
	Game( const Game& ) = delete;
	Game& operator=( const Game& ) = delete;
	void Go();
private:
	void ComposeFrame();
	void UpdateModel();
	/********************************/
	/*  User Functions              */
	bool LoadNewLevel();
	void HandleInput();
	void HandleBall();
	bool LoadBitMapImage(IMAGEINFO& imageData);
	/********************************/
private:
	MainWindow& wnd;
	Graphics gfx;
	/********************************/
	/*  User Variables              */
	std::vector<Ball> m_balls;
	std::vector<Block> m_grid;
	std::vector<std::string> m_levelFilenames;
	Block m_mainBorder;
	Ball* m_ball;
	Paddle m_paddle;
	PlayingField m_field;
	Block* m_lastBlock = nullptr;
	int m_currLevel = 0;
	int m_maxLevels = 5;
	uint m_ballCount;
	Game_State m_gameState = Game_State::intro;
	Vec2f m_maxVelocity = {1.0f,-1.0f};
	float timer = 0.0f;
	const float frame_time = 1.0f / 60.0f;
	IMAGEINFO m_imageData1;
	IMAGEINFO m_imageData2;
	IMAGEINFO m_imageData3;
	std::vector<IMAGEINFO> m_images;
	/********************************/
};